/*
 * burn.c: A plugin for the Video Disk Recorder
 *
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 * $Id: burn.c,v 1.20 2006/07/03 18:18:56 lordjaxom Exp $
 */

#include "burn.h"
#include "i18n.h"
#include "menuburn.h"
#include "setup.h"
#include "skins.h"
#include "manager.h"
#include "logger-vdr.h"
#include "proctools/format.h"
#include <string>
#include <sstream>
#include <vdr/plugin.h>
#include <vdr/videodir.h>
#include <vdr/config.h>

namespace vdr_burn
{

	using namespace std;
	using proctools::format;

	const char* plugin::VERSION        = "0.1.0-pre20";
	const char* plugin::DESCRIPTION    = "Versatile convert-and-burn plugin";
	const char* plugin::MAINMENUENTRY  = "Create DVDs";

	plugin::plugin(void):
			m_mainMenuEntry( tr(MAINMENUENTRY) )
	{
	}

	plugin::~plugin()
	{
	}

	const char *plugin::CommandLineHelp(void)
	{
		static string help_str;

		if (help_str.empty())
			help_str = format(
				 "  -t DIR,   --tempdir=DIR   use DIR for temporary files\n"
				 "                            MUST be on UNIX filesystem\n"
				 "                            (default: {0})\n"
				 "  -d DIR,   --datadir=DIR   use DIR for temporary datafiles\n"
				 "                            (default: {1})\n"
				 "  -D DEV    --dvd=DEV       use DEV as DVD Writer\n"
				 "                            (default: {2})\n"
				 "  -i DIR    --iso=DIR       use DIR to store ISO images\n"
				 "                            (default: {3})\n"
				 ) % BurnParameters.TempPath % BurnParameters.DataPath
				   % BurnParameters.DvdDevice % BurnParameters.IsoPath;

		return help_str.c_str();
	}

	bool plugin::ProcessArgs(int argc, char *argv[])
	{
		return BurnParameters.ProcessArgs(argc, argv);
	}

	bool plugin::Start(void)
	{
		manager::start();
		logger_vdr::start();
		RegisterI18n(i18n::get_phrases());
		return skin_list::get().load(get_config_path());
	}

	void plugin::Stop(void)
	{
		manager::stop();
		logger_vdr::stop();
	}

#ifndef APIVERSION
	bool plugin::Active(void)
#else
	cString plugin::Active(void)
#endif
	{
#ifndef APIVERSION
		return manager::get_is_busy();
#else
		if (manager::get_is_busy())
			return tr("Burn job active");
		return 0;
#endif
	}

	const char *plugin::MainMenuEntry(void)
	{
		ostringstream str;
		str << tr(MAINMENUENTRY);

		if (setup::get().MainMenuStatus) {
			string status( manager::get_status_message() );
			if (!status.empty())
				str << " (" << status << ")";
		}

		m_mainMenuEntry = str.str();
		return m_mainMenuEntry.c_str();
	}

	cOsdObject *plugin::MainMenuAction(void)
	{
		return new menu::starter;
	}

	cMenuSetupPage *plugin::SetupMenu(void)
	{
		return new cBurnMenuSetupPage;
	}

	bool plugin::SetupParse(const char *Name, const char *Value)
	{
		return setup::get().set(Name, Value);
	}

}

VDRPLUGINCREATOR(vdr_burn::plugin); // Don't touch this!
