#include "chain-vdr.h"
#include "chain-dvd.h"
#include "chain-archive.h"
#include "logger-vdr.h"
#include "setup.h"
#include "menuburn.h"
#include "proctools/format.h"
#include "proctools/logger.h"
#include <memory>
#include <vdr/config.h>
#include <sys/time.h>
#include <sys/resource.h>

namespace vdr_burn
{

	using namespace std;
	using proctools::chain;
	using proctools::logger;
	using proctools::format;

	typedef chain_vdr*(*chain_create_t)(job&);

	chain_vdr::chain_vdr(const string& name, job& job_):
			chain(name),
#if VDRVERSNUM >= 10300
			cThread("burn: subprocess watcher"),
#endif
			m_job(job_),
			m_progress(0),
			m_burning(false),
			m_burnProgress(0),
			m_canceled(false)
	{
		const string templ = "/.vdr-burn.XXXXXX";
		m_paths.temp = create_temp_path(BurnParameters.TempPath + templ);
		m_paths.data = create_temp_path(BurnParameters.DataPath + templ);
		job_.set_paths(m_paths);
		logger_vdr::set_logfile( get_log_path() );
	}

	chain_vdr::~chain_vdr()
	{
		stop();
		execute(format("rm -rf {0}") % m_paths.temp);
		execute(format("rm -rf {0}") % m_paths.data);
	}

	chain_vdr* chain_vdr::create_chain(job& job)
	{
		switch (job.get_disk_type()) {
		case disktype_dvd_menu:
		case disktype_dvd_nomenu: return new chain_dvd(job);

		case disktype_archive:    return new chain_archive(job);
		}

		logger::error( "chain_vdr::create_chain requested for unknown disktype" );
		BURN_THROW_DEBUG( "chain_vdr::create_chain requested for unknown disktype" );
		return 0;
	}

	void chain_vdr::stop(void)
	{
		m_canceled = true;
		chain::stop();
		Cancel(3);
	}

	void chain_vdr::add_process(proctools::process* proc)
	{
		proc->set_workdir(m_paths.data);
		chain::add_process(proc);
	}

	string chain_vdr::create_temp_path(const string& templ)
	{
		char *path = new char[templ.length() + 1];
		strcpy(path, templ.c_str());
		mkdtemp(path);

		string result( path );
		delete[] path;
		return result;
	}

	void chain_vdr::all_done(void)
	{
		logger_vdr::close_logfile();

		if (setup::get().PreserveLogFiles) {
			string target = BurnParameters.IsoPath.empty() ? BurnParameters.TempPath : BurnParameters.IsoPath;
			execute( format("cp \"{0}\" \"{1}/vdrburn-{2}.log\"")
					% shell_escape( get_log_path() )
					% shell_escape( target )
					% shell_escape( m_job.get_title() ) );
		}
	}

	void chain_vdr::Action(void)
	{
#if VDRVERSNUM < 10300
		logger::info(format("subprocess watcher thread started (pid={0})")
					 % getpid());
		setpriority(PRIO_PROCESS, 0, 19);
#else
		SetPriority(19);
#endif

		run();

#if VDRVERSNUM < 10300
		logger::info(format("subprocess watcher thread stopped (pid={0})")
					 % getpid());
#endif
	}
};
