/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: i18n.h,v 1.7 2006/06/08 19:06:28 lordjaxom Exp $
 */

#ifndef VDR_BURN_I18N_H
#define VDR_BURN_I18N_H

#include "common.h"
#include <vdr/i18n.h>

namespace vdr_burn
{

	// --- i18n ---------------------------------------------------------------

	// Implemented as a Meyers-Singleton
	class i18n
	{
	private:
		static const tI18nPhrase m_phrases[];

		int m_osdLanguage;

		const char* m_diskTypes[disktype_count];
		const char* m_storeModes[storemode_count];
		const char* m_chaptersModes[chaptersmode_count];
		const char* m_diskSizes[disksize_count];
		const char* m_demuxTypes[demuxtype_count];
		const char* m_requantTypes[requanttype_count];

	protected:
		void prepare(void);

		template<unsigned int N>
		void prepare_strings(const char*(&target)[N], const char*(&source)[N]);

		static i18n& get(void);

	public:
		i18n(void);

		static const tI18nPhrase* get_phrases(void) { return m_phrases; }

		static const char* const* get_disk_types(void);
		static const char* const* get_store_modes(void);
		static const char* const* get_chapters_modes(void);
		static const char* const* get_disk_sizes(void);
		static const char* const* get_demux_types(void);
		static const char* const* get_requant_types(void);
	};

	inline
	const char* const* i18n::get_disk_types(void)
	{
		get().prepare();
		return get().m_diskTypes;
	}

	inline
	const char* const* i18n::get_store_modes(void)
	{
		get().prepare();
		return get().m_storeModes;
	}

	inline
	const char* const* i18n::get_chapters_modes(void)
	{
		get().prepare();
		return get().m_chaptersModes;
	}

	inline
	const char* const* i18n::get_disk_sizes(void)
	{
		get().prepare();
		return get().m_diskSizes;
	}

	inline
	const char* const* i18n::get_demux_types(void)
	{
		get().prepare();
		return get().m_demuxTypes;
	}

	inline
	const char* const* i18n::get_requant_types(void)
	{
		get().prepare();
		return get().m_requantTypes;
	}
	
	template<unsigned int N>
	void i18n::prepare_strings(const char*(&target)[N], const char*(&source)[N])
	{
		for (unsigned int i = 0; i < N; ++i)
			target[i] = tr(source[i]);
	}

}

#endif // VDR_BURN_I18N_H
