/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: menuitems.c,v 1.12 2006/06/29 18:18:05 lordjaxom Exp $
 */

#include "menuitems.h"
#include "jobs.h"
#include "i18n.h"
#include "common.h"
#include "proctools/format.h"
#include "boost/bind.hpp"
#include <algorithm>
#include <vdr/remote.h>
#include <vdr/status.h>
#include <ctype.h>

#define SHIFTTIMEOUT 2

namespace vdr_burn
{

	using namespace std;
	using proctools::format;
	using boost::bind;

	namespace menu
	{

		// --- text_item ------------------------------------------------------

		text_item::text_item(const std::string& text, bool selectable):
				cOsdItem( text.c_str() )
		{
			SetSelectable(selectable);
		}

		eOSState text_item::ProcessKey(eKeys key)
		{
			eKeys plainKey( static_cast<eKeys>(key & ~k_Flags) );
			return plainKey == kLeft || plainKey == kRight
				? osContinue
				: cOsdItem::ProcessKey(key);
		}

		// --- job_item -------------------------------------------------------

		job_item::job_item(job* job_, int number, bool done):
				cOsdItem(),
				m_job(job_)
		{
			SetText(cString::sprintf("%d. %s", number, m_job->get_title()));
		}

		// --- recording_item -------------------------------------------------

		recording_item::recording_item(const cRecording* recording_, int level):
				cOsdItem(get_recording_title(recording_, level).c_str()),
				m_recording(recording_),
				m_total(0),
				m_new(0)
		{
			if (level < recording_->HierarchyLevels())
				m_name = Text() + 2;
		}

		void recording_item::add_entry(bool isNew)
		{
			++m_total;
			if (isNew)
				++m_new;
			SetText(cString::sprintf("%d\t%d\t%s", m_total, m_new,
									 m_name.c_str()));
		}

		// --- recording_edit_item --------------------------------------------

		recording_edit_item::recording_edit_item(const recording_list::iterator& rec):
				cMenuEditStrItem( rec->get_datetime().c_str(), rec->m_name, JOBNAMELEN,
						TitleChars ),
				m_recording( rec )
		{
		}

		// --- speed_edit_item ------------------------------------------------

		speed_edit_item::speed_edit_item(const string& name, int& value):
				cMenuEditIntItem(name.c_str(), &value, 0, 32)
		{
			Set();
		}

		void speed_edit_item::Set(void)
		{
			if (*value == 0)
				SetValue(tr("unlimited"));
			else
				cMenuEditIntItem::Set();
		}

		// --- language_item --------------------------------------------------

		language_item::language_item(const std::string& name, std::string& language):
				strings_t( track_info::get_language_codes() ),
				value_t( strings_t::value.find_index(language) ),
 				cMenuEditStraItem( name.c_str(), value_t::get_pointer(), strings_t::value.size(), strings_t::value.get_strings() ),
				m_language( language )
		{
		}

		eOSState language_item::ProcessKey(eKeys key)
		{
			int oldIndex = value_t::value;
			eOSState state = cMenuEditStraItem::ProcessKey(key);
			if (oldIndex != value_t::value)
				m_language = strings_t::value.get_strings()[value_t::value];
			return state;
		}

		// --- aspect_item ----------------------------------------------------

		aspect_item::aspect_item(const std::string& name, track_info::aspectratio& aspect):
				value_t( aspect != track_info::aspectratio_4_3 ),
				bool_item( name.c_str(), value_t::value, "4:3", "16:9" ),
				m_aspect( aspect )
		{
		}

		eOSState aspect_item::ProcessKey(eKeys key)
		{
			int oldValue = value_t::value;
			eOSState state = bool_item::ProcessKey(key);
			if (oldValue != value_t::value)
				m_aspect = value_t::value ? track_info::aspectratio_16_9 : track_info::aspectratio_4_3;
			return state;
		}

	}
}
