/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: menuitems.h,v 1.10 2006/06/29 18:18:05 lordjaxom Exp $
 */

#ifndef VDR_BURN_MENUITEMS_H
#define VDR_BURN_MENUITEMS_H

#include "common.h"
#include "jobs.h"
#include <algorithm>
#include <iterator>
#include <memory>
#include <string>
#include <vector>
#include "boost/bind.hpp"
#include <vdr/menuitems.h>

namespace vdr_burn
{

	namespace menu
	{

		// --- bool_item ------------------------------------------------------

		class bool_item: public cMenuEditBoolItem
		{
		public:
			bool_item(const std::string& text, int& data, const char* falseString = 0, const char* trueString = 0):
				cMenuEditBoolItem(text.c_str(), &data, falseString, trueString) {}
		};

		// --- text_item ------------------------------------------------------

		class text_item: public cOsdItem
		{
		public:
			text_item(const std::string& text = "", bool selectable = false);

		protected:
			virtual eOSState ProcessKey(eKeys key);
		};

		// --- job ------------------------------------------------------------

		class job_item: public cOsdItem
		{
		public:
			job_item(job* job_, int number, bool done = false);

			job* get_job(void) const { return m_job; }

		private:
			job* m_job;
		};

		// --- recording_item -------------------------------------------------

		class recording_item: public cOsdItem
		{
		private:
			const cRecording* m_recording;
			std::string       m_name;
			int               m_total;
			int               m_new;

		public:
			recording_item(const cRecording *recording_, int level);

			void add_entry(bool isNew);

			const std::string& get_name(void) const { return m_name; }
			const cRecording* get_recording(void) const { return m_recording; }
			bool is_directory(void) const { return !m_name.empty(); }
		};


		// --- recording_edit_item --------------------------------------------

		class recording_edit_item: public cMenuEditStrItem
		{
		public:
			recording_edit_item(const recording_list::iterator& rec);

			const recording_list::iterator& get_recording(void) const;

		private:
			recording_list::iterator m_recording;
		};

		inline
		const recording_list::iterator& recording_edit_item::get_recording(void) const
		{
			return m_recording;
		}

		// --- speed_edit_item ------------------------------------------------

		class speed_edit_item: public cMenuEditIntItem
		{
		public:
			speed_edit_item(const std::string& name, int& value);

		protected:
			virtual void Set(void);
		};

		// --- language_item --------------------------------------------------

		class language_item: public base_from_member<string_list, 1>,
							 public base_from_member<int, 0>,
							 public cMenuEditStraItem
		{
			typedef base_from_member<int, 0> value_t;
			typedef base_from_member<string_list, 1> strings_t;

		public:
			language_item(const std::string& name, std::string& language);

		protected:
			virtual eOSState ProcessKey(eKeys key);

		private:
			std::string& m_language;
		};

		// --- aspect_item ----------------------------------------------------

		class aspect_item: public base_from_member<int, 0>, public bool_item
		{
			typedef base_from_member<int, 0> value_t;

		public:
			aspect_item(const std::string& name, track_info::aspectratio& aspect);

		protected:
			virtual eOSState ProcessKey(eKeys key);

		private:
			track_info::aspectratio& m_aspect;
		};
	}
}

#endif // VDR_BURN_MENUITEMS_H
